import { MovedText } from './linesDiffComputer.js';
import { DetailedLineRangeMapping } from './rangeMapping.js';
import '../../editor.api.js';

/**
 * Options for the diff computation.
 * @internal
 */
interface IDocumentDiffProviderOptions {
    /**
     * When set to true, the diff should ignore whitespace changes.
     */
    ignoreTrimWhitespace: boolean;
    /**
     * A diff computation should throw if it takes longer than this value.
     */
    maxComputationTimeMs: number;
    /**
     * If set, the diff computation should compute moves in addition to insertions and deletions.
     */
    computeMoves: boolean;
}
/**
 * Represents a diff between two text models.
 * @internal
 */
interface IDocumentDiff {
    /**
     * If true, both text models are identical (byte-wise).
     */
    readonly identical: boolean;
    /**
     * If true, the diff computation timed out and the diff might not be accurate.
     */
    readonly quitEarly: boolean;
    /**
     * Maps all modified line ranges in the original to the corresponding line ranges in the modified text model.
     */
    readonly changes: readonly DetailedLineRangeMapping[];
    /**
     * Sorted by original line ranges.
     * The original line ranges and the modified line ranges must be disjoint (but can be touching).
     */
    readonly moves: readonly MovedText[];
}

export type { IDocumentDiff, IDocumentDiffProviderOptions };
