import { Registry } from '../../platform/registry/common/platform.js';
import { localizeWithPath } from '../../nls.js';
import { Extensions as Extensions$1 } from '../../platform/configuration/common/configurationRegistry.js';
import { isMacintosh, isWeb, isWindows, isLinux } from '../../base/common/platform.js';
import { ConfigurationMigrationWorkbenchContribution, DynamicWorkbenchSecurityConfiguration, workbenchConfigurationNodeBase, problemsConfigurationNodeBase, Extensions } from '../common/configuration.js';
import { isStandalone } from '../../base/browser/browser.js';
import { registerWorkbenchContribution2 } from '../common/contributions.js';
import { defaultWindowTitle, defaultWindowTitleSeparator } from './parts/titlebar/windowTitle.js';
import { CustomEditorLabelService } from '../services/editor/common/customEditorLabelService.js';
const registry = ( Registry.as(Extensions$1.Configuration));
(function registerConfiguration() {
    registerWorkbenchContribution2(ConfigurationMigrationWorkbenchContribution.ID, ConfigurationMigrationWorkbenchContribution, 4 );
    registerWorkbenchContribution2(DynamicWorkbenchSecurityConfiguration.ID, DynamicWorkbenchSecurityConfiguration, 3 );
    registry.registerConfiguration({
        ...workbenchConfigurationNodeBase,
        'properties': {
            'workbench.editor.titleScrollbarSizing': {
                type: 'string',
                enum: ['default', 'large'],
                enumDescriptions: [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.titleScrollbarSizing.default',
                        "The default size."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.titleScrollbarSizing.large',
                        "Increases the size, so it can be grabbed more easily with the mouse."
                    ))
                ],
                description: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'tabScrollbarHeight',
                    "Controls the height of the scrollbars used for tabs and breadcrumbs in the editor title area."
                )),
                default: 'default',
            },
            ["workbench.editor.showTabs" ]: {
                'type': 'string',
                'enum': ["multiple" , "single" , "none" ],
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.showTabs.multiple',
                        "Each editor is displayed as a tab in the editor title area."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.showTabs.single',
                        "The active editor is displayed as a single large tab in the editor title area."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.showTabs.none',
                        "The editor title area is not displayed."
                    )),
                ],
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'showEditorTabs',
                    "Controls whether opened editors should show as individual tabs, one single large tab or if the title area should not be shown."
                )),
                'default': 'multiple'
            },
            ["workbench.editor.editorActionsLocation" ]: {
                'type': 'string',
                'enum': ["default" , "titleBar" , "hidden" ],
                'markdownEnumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        { comment: ['{0} will be a setting name rendered as a link'], key: 'workbench.editor.editorActionsLocation.default' },
                        "Show editor actions in the window title bar when {0} is set to {1}. Otherwise, editor actions are shown in the editor tab bar.",
                        '`#workbench.editor.showTabs#`',
                        '`none`'
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        { comment: ['{0} will be a setting name rendered as a link'], key: 'workbench.editor.editorActionsLocation.titleBar' },
                        "Show editor actions in the window title bar. If {0} is set to {1}, editor actions are hidden.",
                        '`#window.customTitleBarVisibility#`',
                        '`never`'
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.editorActionsLocation.hidden',
                        "Editor actions are not shown."
                    )),
                ],
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'editorActionsLocation',
                    "Controls where the editor actions are shown."
                )),
                'default': 'default'
            },
            'workbench.editor.wrapTabs': {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'wrapTabs' },
                    "Controls whether tabs should be wrapped over multiple lines when exceeding available space or whether a scrollbar should appear instead. This value is ignored when {0} is not set to '{1}'.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                )),
                'default': false
            },
            'workbench.editor.scrollToSwitchTabs': {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'scrollToSwitchTabs' },
                    "Controls whether scrolling over tabs will open them or not. By default tabs will only reveal upon scrolling, but not open. You can press and hold the Shift-key while scrolling to change this behavior for that duration. This value is ignored when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                )),
                'default': false
            },
            'workbench.editor.highlightModifiedTabs': {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'highlightModifiedTabs' },
                    "Controls whether a top border is drawn on tabs for editors that have unsaved changes. This value is ignored when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    `multiple`
                )),
                'default': false
            },
            'workbench.editor.decorations.badges': {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'decorations.badges',
                    "Controls whether editor file decorations should use badges."
                )),
                'default': true
            },
            'workbench.editor.decorations.colors': {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'decorations.colors',
                    "Controls whether editor file decorations should use colors."
                )),
                'default': true
            },
            [CustomEditorLabelService.SETTING_ID_ENABLED]: {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.editor.label.enabled',
                    "Controls whether the custom workbench editor labels should be applied."
                )),
                'default': true,
            },
            [CustomEditorLabelService.SETTING_ID_PATTERNS]: {
                'type': 'object',
                'markdownDescription': (() => {
                    let customEditorLabelDescription = ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.label.patterns',
                        "Controls the rendering of the editor label. Each __Item__ is a pattern that matches a file path. Both relative and absolute file paths are supported. In case multiple patterns match, the longest matching path will be picked. Each __Value__ is the template for the rendered editor when the __Item__ matches. Variables are substituted based on the context:"
                    ));
                    customEditorLabelDescription += '\n- ' + [
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'workbench.editor.label.dirname',
                            "`${dirname}`: name of the folder in which the file is located (e.g. `root/folder/file.txt -> folder`)."
                        )),
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'workbench.editor.label.nthdirname',
                            "`${dirname(N)}`: name of the nth parent folder in which the file is located (e.g. `N=1: root/folder/file.txt -> root`)."
                        )),
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'workbench.editor.label.filename',
                            "`${filename}`: name of the file without the file extension (e.g. `root/folder/file.txt -> file`)."
                        )),
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'workbench.editor.label.extname',
                            "`${extname}`: the file extension (e.g. `root/folder/file.txt -> txt`)."
                        )),
                    ].join('\n- ');
                    customEditorLabelDescription += '\n\n' + ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'customEditorLabelDescriptionExample',
                        "Example: `\"**/static/**/*.html\": \"${filename} - ${dirname} (${extname})\"` will render a file `root/static/folder/file.html` as `file - folder (html)`."
                    ));
                    return customEditorLabelDescription;
                })(),
                additionalProperties: {
                    type: 'string',
                    markdownDescription: ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.label.template',
                        "The template which should be rendered when the pattern mtches. May include the variables ${dirname}, ${filename} and ${extname}."
                    )),
                    minLength: 1,
                    pattern: '.*[a-zA-Z0-9].*'
                },
                'default': {}
            },
            'workbench.editor.labelFormat': {
                'type': 'string',
                'enum': ['default', 'short', 'medium', 'long'],
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.labelFormat.default',
                        "Show the name of the file. When tabs are enabled and two files have the same name in one group the distinguishing sections of each file's path are added. When tabs are disabled, the path relative to the workspace folder is shown if the editor is active."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.labelFormat.short',
                        "Show the name of the file followed by its directory name."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.labelFormat.medium',
                        "Show the name of the file followed by its path relative to the workspace folder."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.labelFormat.long',
                        "Show the name of the file followed by its absolute path."
                    ))
                ],
                'default': 'default',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'tabDescription',
                    "Controls the format of the label for an editor."
                )),
            },
            'workbench.editor.untitled.labelFormat': {
                'type': 'string',
                'enum': ['content', 'name'],
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.untitled.labelFormat.content',
                        "The name of the untitled file is derived from the contents of its first line unless it has an associated file path. It will fallback to the name in case the line is empty or contains no word characters."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.untitled.labelFormat.name',
                        "The name of the untitled file is not derived from the contents of the file."
                    )),
                ],
                'default': 'content',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'untitledLabelFormat',
                    "Controls the format of the label for an untitled editor."
                )),
            },
            'workbench.editor.empty.hint': {
                'type': 'string',
                'enum': ['text', 'hidden'],
                'default': 'text',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    "workbench.editor.empty.hint",
                    "Controls if the empty editor text hint should be visible in the editor."
                ))
            },
            'workbench.editor.languageDetection': {
                type: 'boolean',
                default: true,
                description: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.editor.languageDetection',
                    "Controls whether the language in a text editor is automatically detected unless the language has been explicitly set by the language picker. This can also be scoped by language so you can specify which languages you do not want to be switched off of. This is useful for languages like Markdown that often contain other languages that might trick language detection into thinking it's the embedded language and not Markdown."
                )),
                scope: 5
            },
            'workbench.editor.historyBasedLanguageDetection': {
                type: 'boolean',
                default: true,
                tags: ['experimental'],
                description: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.editor.historyBasedLanguageDetection',
                    "Enables use of editor history in language detection. This causes automatic language detection to favor languages that have been recently opened and allows for automatic language detection to operate with smaller inputs."
                )),
            },
            'workbench.editor.preferHistoryBasedLanguageDetection': {
                type: 'boolean',
                default: false,
                tags: ['experimental'],
                description: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.editor.preferBasedLanguageDetection',
                    "When enabled, a language detection model that takes into account editor history will be given higher precedence."
                )),
            },
            'workbench.editor.languageDetectionHints': {
                type: 'object',
                default: { 'untitledEditors': true, 'notebookEditors': true },
                tags: ['experimental'],
                description: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.editor.showLanguageDetectionHints',
                    "When enabled, shows a Status bar Quick Fix when the editor language doesn't match detected content language."
                )),
                additionalProperties: false,
                properties: {
                    untitledEditors: {
                        type: 'boolean',
                        description: ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'workbench.editor.showLanguageDetectionHints.editors',
                            "Show in untitled text editors"
                        )),
                    },
                    notebookEditors: {
                        type: 'boolean',
                        description: ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'workbench.editor.showLanguageDetectionHints.notebook',
                            "Show in notebook editors"
                        )),
                    }
                }
            },
            'workbench.editor.tabActionLocation': {
                type: 'string',
                enum: ['left', 'right'],
                default: 'right',
                markdownDescription: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0} will be a setting name rendered as a link'], key: 'tabActionLocation' },
                    "Controls the position of the editor's tabs action buttons (close, unpin). This value is ignored when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                ))
            },
            'workbench.editor.tabActionCloseVisibility': {
                type: 'boolean',
                default: true,
                description: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.editor.tabActionCloseVisibility',
                    "Controls the visibility of the tab close action button."
                ))
            },
            'workbench.editor.tabActionUnpinVisibility': {
                type: 'boolean',
                default: true,
                description: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.editor.tabActionUnpinVisibility',
                    "Controls the visibility of the tab unpin action button."
                ))
            },
            'workbench.editor.tabSizing': {
                'type': 'string',
                'enum': ['fit', 'shrink', 'fixed'],
                'default': 'fit',
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.tabSizing.fit',
                        "Always keep tabs large enough to show the full editor label."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.tabSizing.shrink',
                        "Allow tabs to get smaller when the available space is not enough to show all tabs at once."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.tabSizing.fixed',
                        "Make all tabs the same size, while allowing them to get smaller when the available space is not enough to show all tabs at once."
                    ))
                ],
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'tabSizing' },
                    "Controls the size of editor tabs. This value is ignored when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                ))
            },
            'workbench.editor.tabSizingFixedMinWidth': {
                'type': 'number',
                'default': 50,
                'minimum': 38,
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'workbench.editor.tabSizingFixedMinWidth' },
                    "Controls the minimum width of tabs when {0} size is set to {1}.",
                    '`#workbench.editor.tabSizing#`',
                    '`fixed`'
                ))
            },
            'workbench.editor.tabSizingFixedMaxWidth': {
                'type': 'number',
                'default': 160,
                'minimum': 38,
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'workbench.editor.tabSizingFixedMaxWidth' },
                    "Controls the maximum width of tabs when {0} size is set to {1}.",
                    '`#workbench.editor.tabSizing#`',
                    '`fixed`'
                ))
            },
            'window.density.editorTabHeight': {
                'type': 'string',
                'enum': ['default', 'compact'],
                'default': 'default',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'workbench.editor.tabHeight' },
                    "Controls the height of editor tabs. Also applies to the title control bar when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                ))
            },
            'workbench.editor.pinnedTabSizing': {
                'type': 'string',
                'enum': ['normal', 'compact', 'shrink'],
                'default': 'normal',
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.pinnedTabSizing.normal',
                        "A pinned tab inherits the look of non pinned tabs."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.pinnedTabSizing.compact',
                        "A pinned tab will show in a compact form with only icon or first letter of the editor name."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.pinnedTabSizing.shrink',
                        "A pinned tab shrinks to a compact fixed size showing parts of the editor name."
                    ))
                ],
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'pinnedTabSizing' },
                    "Controls the size of pinned editor tabs. Pinned tabs are sorted to the beginning of all opened tabs and typically do not close until unpinned. This value is ignored when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                ))
            },
            'workbench.editor.pinnedTabsOnSeparateRow': {
                'type': 'boolean',
                'default': false,
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'workbench.editor.pinnedTabsOnSeparateRow' },
                    "When enabled, displays pinned tabs in a separate row above all other tabs. This value is ignored when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                )),
            },
            'workbench.editor.preventPinnedEditorClose': {
                'type': 'string',
                'enum': ['keyboardAndMouse', 'keyboard', 'mouse', 'never'],
                'default': 'keyboardAndMouse',
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.preventPinnedEditorClose.always',
                        "Always prevent closing the pinned editor when using mouse middle click or keyboard."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.preventPinnedEditorClose.onlyKeyboard',
                        "Prevent closing the pinned editor when using the keyboard."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.preventPinnedEditorClose.onlyMouse',
                        "Prevent closing the pinned editor when using mouse middle click."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.preventPinnedEditorClose.never',
                        "Never prevent closing a pinned editor."
                    ))
                ],
                description: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.editor.preventPinnedEditorClose',
                    "Controls whether pinned editors should close when keyboard or middle mouse click is used for closing."
                )),
            },
            'workbench.editor.splitSizing': {
                'type': 'string',
                'enum': ['auto', 'distribute', 'split'],
                'default': 'auto',
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.splitSizingAuto',
                        "Splits the active editor group to equal parts, unless all editor groups are already in equal parts. In that case, splits all the editor groups to equal parts."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.splitSizingDistribute',
                        "Splits all the editor groups to equal parts."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.splitSizingSplit',
                        "Splits the active editor group to equal parts."
                    ))
                ],
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'splitSizing',
                    "Controls the size of editor groups when splitting them."
                ))
            },
            'workbench.editor.splitOnDragAndDrop': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'splitOnDragAndDrop',
                    "Controls if editor groups can be split from drag and drop operations by dropping an editor or file on the edges of the editor area."
                ))
            },
            'workbench.editor.dragToOpenWindow': {
                'type': 'boolean',
                'default': true,
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'dragToOpenWindow',
                    "Controls if editors can be dragged out of the window to open them in a new window. Press and hold the `Alt` key while dragging to toggle this dynamically."
                ))
            },
            'workbench.editor.focusRecentEditorAfterClose': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'focusRecentEditorAfterClose',
                    "Controls whether editors are closed in most recently used order or from left to right."
                )),
                'default': true
            },
            'workbench.editor.showIcons': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'showIcons',
                    "Controls whether opened editors should show with an icon or not. This requires a file icon theme to be enabled as well."
                )),
                'default': true
            },
            'workbench.editor.enablePreview': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'enablePreview',
                    "Controls whether opened editors show as preview editors. Preview editors do not stay open, are reused until explicitly set to be kept open (via double-click or editing), and show file names in italics."
                )),
                'default': true
            },
            'workbench.editor.enablePreviewFromQuickOpen': {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'enablePreviewFromQuickOpen' },
                    "Controls whether editors opened from Quick Open show as preview editors. Preview editors do not stay open, and are reused until explicitly set to be kept open (via double-click or editing). When enabled, hold Ctrl before selection to open an editor as a non-preview. This value is ignored when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                )),
                'default': false
            },
            'workbench.editor.enablePreviewFromCodeNavigation': {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'enablePreviewFromCodeNavigation' },
                    "Controls whether editors remain in preview when a code navigation is started from them. Preview editors do not stay open, and are reused until explicitly set to be kept open (via double-click or editing). This value is ignored when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                )),
                'default': false
            },
            'workbench.editor.closeOnFileDelete': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'closeOnFileDelete',
                    "Controls whether editors showing a file that was opened during the session should close automatically when getting deleted or renamed by some other process. Disabling this will keep the editor open  on such an event. Note that deleting from within the application will always close the editor and that editors with unsaved changes will never close to preserve your data."
                )),
                'default': false
            },
            'workbench.editor.openPositioning': {
                'type': 'string',
                'enum': ['left', 'right', 'first', 'last'],
                'default': 'right',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1}, {2}, {3} will be a setting name rendered as a link'], key: 'editorOpenPositioning' },
                    "Controls where editors open. Select {0} or {1} to open editors to the left or right of the currently active one. Select {2} or {3} to open editors independently from the currently active one.",
                    '`left`',
                    '`right`',
                    '`first`',
                    '`last`'
                ))
            },
            'workbench.editor.openSideBySideDirection': {
                'type': 'string',
                'enum': ['right', 'down'],
                'default': 'right',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'sideBySideDirection',
                    "Controls the default direction of editors that are opened side by side (for example, from the Explorer). By default, editors will open on the right hand side of the currently active one. If changed to `down`, the editors will open below the currently active one."
                ))
            },
            'workbench.editor.closeEmptyGroups': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'closeEmptyGroups',
                    "Controls the behavior of empty editor groups when the last tab in the group is closed. When enabled, empty groups will automatically close. When disabled, empty groups will remain part of the grid."
                )),
                'default': true
            },
            'workbench.editor.revealIfOpen': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'revealIfOpen',
                    "Controls whether an editor is revealed in any of the visible groups if opened. If disabled, an editor will prefer to open in the currently active editor group. If enabled, an already opened editor will be revealed instead of opened again in the currently active editor group. Note that there are some cases where this setting is ignored, such as when forcing an editor to open in a specific group or to the side of the currently active group."
                )),
                'default': false
            },
            'workbench.editor.mouseBackForwardToNavigate': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'mouseBackForwardToNavigate',
                    "Enables the use of mouse buttons four and five for commands 'Go Back' and 'Go Forward'."
                )),
                'default': true
            },
            'workbench.editor.navigationScope': {
                'type': 'string',
                'enum': ['default', 'editorGroup', 'editor'],
                'default': 'default',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'navigationScope',
                    "Controls the scope of history navigation in editors for commands such as 'Go Back' and 'Go Forward'."
                )),
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.navigationScopeDefault',
                        "Navigate across all opened editors and editor groups."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.navigationScopeEditorGroup',
                        "Navigate only in editors of the active editor group."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.navigationScopeEditor',
                        "Navigate only in the active editor."
                    ))
                ],
            },
            'workbench.editor.restoreViewState': {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'restoreViewState',
                    "Restores the last editor view state (such as scroll position) when re-opening editors after they have been closed. Editor view state is stored per editor group and discarded when a group closes. Use the {0} setting to use the last known view state across all editor groups in case no previous view state was found for a editor group.",
                    '`#workbench.editor.sharedViewState#`'
                )),
                'default': true,
                'scope': 5
            },
            'workbench.editor.sharedViewState': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'sharedViewState',
                    "Preserves the most recent editor view state (such as scroll position) across all editor groups and restores that if no specific editor view state is found for the editor group."
                )),
                'default': false
            },
            'workbench.editor.splitInGroupLayout': {
                'type': 'string',
                'enum': ['vertical', 'horizontal'],
                'default': 'horizontal',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'splitInGroupLayout',
                    "Controls the layout for when an editor is split in an editor group to be either vertical or horizontal."
                )),
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.splitInGroupLayoutVertical',
                        "Editors are positioned from top to bottom."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.splitInGroupLayoutHorizontal',
                        "Editors are positioned from left to right."
                    ))
                ]
            },
            'workbench.editor.centeredLayoutAutoResize': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'centeredLayoutAutoResize',
                    "Controls if the centered layout should automatically resize to maximum width when more than one group is open. Once only one group is open it will resize back to the original centered width."
                ))
            },
            'workbench.editor.centeredLayoutFixedWidth': {
                'type': 'boolean',
                'default': false,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'centeredLayoutDynamicWidth',
                    "Controls whether the centered layout tries to maintain constant width when the window is resized."
                ))
            },
            'workbench.editor.doubleClickTabToToggleEditorGroupSizes': {
                'type': 'string',
                'enum': ['maximize', 'expand', 'off'],
                'default': 'expand',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['{0}, {1} will be a setting name rendered as a link'], key: 'doubleClickTabToToggleEditorGroupSizes' },
                    "Controls how the editor group is resized when double clicking on a tab. This value is ignored when {0} is not set to {1}.",
                    '`#workbench.editor.showTabs#`',
                    '`multiple`'
                )),
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.doubleClickTabToToggleEditorGroupSizes.maximize',
                        "All other editor groups are hidden and the current editor group is maximized to take up the entire editor area."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.doubleClickTabToToggleEditorGroupSizes.expand',
                        "The editor group takes as much space as possible by making all other editor groups as small as possible."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.editor.doubleClickTabToToggleEditorGroupSizes.off',
                        "No editor group is resized when double clicking on a tab."
                    ))
                ]
            },
            'workbench.editor.limit.enabled': {
                'type': 'boolean',
                'default': false,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'limitEditorsEnablement',
                    "Controls if the number of opened editors should be limited or not. When enabled, less recently used editors will close to make space for newly opening editors."
                ))
            },
            'workbench.editor.limit.value': {
                'type': 'number',
                'default': 10,
                'exclusiveMinimum': 0,
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'limitEditorsMaximum',
                    "Controls the maximum number of opened editors. Use the {0} setting to control this limit per editor group or across all groups.",
                    '`#workbench.editor.limit.perEditorGroup#`'
                ))
            },
            'workbench.editor.limit.excludeDirty': {
                'type': 'boolean',
                'default': false,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'limitEditorsExcludeDirty',
                    "Controls if the maximum number of opened editors should exclude dirty editors for counting towards the configured limit."
                ))
            },
            'workbench.editor.limit.perEditorGroup': {
                'type': 'boolean',
                'default': false,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'perEditorGroup',
                    "Controls if the limit of maximum opened editors should apply per editor group or across all editor groups."
                ))
            },
            'workbench.localHistory.enabled': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'localHistoryEnabled',
                    "Controls whether local file history is enabled. When enabled, the file contents of an editor that is saved will be stored to a backup location to be able to restore or review the contents later. Changing this setting has no effect on existing local file history entries."
                )),
                'scope': 4
            },
            'workbench.localHistory.maxFileSize': {
                'type': 'number',
                'default': 256,
                'minimum': 1,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'localHistoryMaxFileSize',
                    "Controls the maximum size of a file (in KB) to be considered for local file history. Files that are larger will not be added to the local file history. Changing this setting has no effect on existing local file history entries."
                )),
                'scope': 4
            },
            'workbench.localHistory.maxFileEntries': {
                'type': 'number',
                'default': 50,
                'minimum': 0,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'localHistoryMaxFileEntries',
                    "Controls the maximum number of local file history entries per file. When the number of local file history entries exceeds this number for a file, the oldest entries will be discarded."
                )),
                'scope': 4
            },
            'workbench.localHistory.exclude': {
                'type': 'object',
                'patternProperties': {
                    '.*': { 'type': 'boolean' }
                },
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'exclude',
                    "Configure paths or [glob patterns](https://aka.ms/vscode-glob-patterns) for excluding files from the local file history. Glob patterns are always evaluated relative to the path of the workspace folder unless they are absolute paths. Changing this setting has no effect on existing local file history entries."
                )),
                'scope': 4
            },
            'workbench.localHistory.mergeWindow': {
                'type': 'number',
                'default': 10,
                'minimum': 1,
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'mergeWindow',
                    "Configure an interval in seconds during which the last entry in local file history is replaced with the entry that is being added. This helps reduce the overall number of entries that are added, for example when auto save is enabled. This setting is only applied to entries that have the same source of origin. Changing this setting has no effect on existing local file history entries."
                )),
                'scope': 4
            },
            'workbench.commandPalette.history': {
                'type': 'number',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'commandHistory',
                    "Controls the number of recently used commands to keep in history for the command palette. Set to 0 to disable command history."
                )),
                'default': 50,
                'minimum': 0
            },
            'workbench.commandPalette.preserveInput': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'preserveInput',
                    "Controls whether the last typed input to the command palette should be restored when opening it the next time."
                )),
                'default': false
            },
            'workbench.commandPalette.experimental.suggestCommands': {
                'type': 'boolean',
                tags: ['experimental'],
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'suggestCommands',
                    "Controls whether the command palette should have a list of commonly used commands."
                )),
                'default': false
            },
            'workbench.commandPalette.experimental.askChatLocation': {
                'type': 'string',
                tags: ['experimental'],
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'askChatLocation',
                    "Controls where the command palette should ask chat questions."
                )),
                'default': 'chatView',
                enum: ['chatView', 'quickChat'],
                enumDescriptions: [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'askChatLocation.chatView',
                        "Ask chat questions in the Chat view."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'askChatLocation.quickChat',
                        "Ask chat questions in Quick Chat."
                    ))
                ]
            },
            'workbench.commandPalette.experimental.enableNaturalLanguageSearch': {
                'type': 'boolean',
                tags: ['experimental'],
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'enableNaturalLanguageSearch',
                    "Controls whether the command palette should include similar commands. You must have an extension installed that provides Natural Language support."
                )),
                'default': true
            },
            'workbench.quickOpen.closeOnFocusLost': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'closeOnFocusLost',
                    "Controls whether Quick Open should close automatically once it loses focus."
                )),
                'default': true
            },
            'workbench.quickOpen.preserveInput': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.quickOpen.preserveInput',
                    "Controls whether the last typed input to Quick Open should be restored when opening it the next time."
                )),
                'default': false
            },
            'workbench.settings.openDefaultSettings': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'openDefaultSettings',
                    "Controls whether opening settings also opens an editor showing all default settings."
                )),
                'default': false
            },
            'workbench.settings.useSplitJSON': {
                'type': 'boolean',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'useSplitJSON',
                    "Controls whether to use the split JSON editor when editing settings as JSON."
                )),
                'default': false
            },
            'workbench.settings.openDefaultKeybindings': {
                'type': 'boolean',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'openDefaultKeybindings',
                    "Controls whether opening keybinding settings also opens an editor showing all default keybindings."
                )),
                'default': false
            },
            'workbench.sideBar.location': {
                'type': 'string',
                'enum': ['left', 'right'],
                'default': 'left',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'sideBarLocation',
                    "Controls the location of the primary side bar and activity bar. They can either show on the left or right of the workbench. The secondary side bar will show on the opposite side of the workbench."
                ))
            },
            'workbench.panel.defaultLocation': {
                'type': 'string',
                'enum': ['left', 'bottom', 'right'],
                'default': 'bottom',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'panelDefaultLocation',
                    "Controls the default location of the panel (Terminal, Debug Console, Output, Problems) in a new workspace. It can either show at the bottom, right, or left of the editor area."
                )),
            },
            'workbench.panel.opensMaximized': {
                'type': 'string',
                'enum': ['always', 'never', 'preserve'],
                'default': 'preserve',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'panelOpensMaximized',
                    "Controls whether the panel opens maximized. It can either always open maximized, never open maximized, or open to the last state it was in before being closed."
                )),
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.panel.opensMaximized.always',
                        "Always maximize the panel when opening it."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.panel.opensMaximized.never',
                        "Never maximize the panel when opening it. The panel will open un-maximized."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.panel.opensMaximized.preserve',
                        "Open the panel to the state that it was in, before it was closed."
                    ))
                ]
            },
            'workbench.statusBar.visible': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'statusBarVisibility',
                    "Controls the visibility of the status bar at the bottom of the workbench."
                ))
            },
            ["workbench.activityBar.location" ]: {
                'type': 'string',
                'enum': ['default', 'top', 'bottom', 'hidden'],
                'default': 'default',
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    { comment: ['This is the description for a setting'], key: 'activityBarLocation' },
                    "Controls the location of the Activity Bar. It can either show to the `default` or `top` / `bottom` of the Primary and Secondary Side Bar or `hidden`."
                )),
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.activityBar.location.default',
                        "Show the Activity Bar of the Primary Side Bar on the side."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.activityBar.location.top',
                        "Show the Activity Bar on top of the Primary and Secondary Side Bar."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.activityBar.location.bottom',
                        "Show the Activity Bar at the bottom of the Primary and Secondary Side Bar."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.activityBar.location.hide',
                        "Hide the Activity Bar in the Primary and Secondary Side Bar."
                    ))
                ],
            },
            'workbench.activityBar.iconClickBehavior': {
                'type': 'string',
                'enum': ['toggle', 'focus'],
                'default': 'toggle',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'activityBarIconClickBehavior',
                    "Controls the behavior of clicking an activity bar icon in the workbench."
                )),
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.activityBar.iconClickBehavior.toggle',
                        "Hide the side bar if the clicked item is already visible."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.activityBar.iconClickBehavior.focus',
                        "Focus side bar if the clicked item is already visible."
                    ))
                ]
            },
            'workbench.view.alwaysShowHeaderActions': {
                'type': 'boolean',
                'default': false,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'viewVisibility',
                    "Controls the visibility of view header actions. View header actions may either be always visible, or only visible when that view is focused or hovered over."
                ))
            },
            'workbench.fontAliasing': {
                'type': 'string',
                'enum': ['default', 'antialiased', 'none', 'auto'],
                'default': 'default',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'fontAliasing',
                    "Controls font aliasing method in the workbench."
                )),
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.fontAliasing.default',
                        "Sub-pixel font smoothing. On most non-retina displays this will give the sharpest text."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.fontAliasing.antialiased',
                        "Smooth the font on the level of the pixel, as opposed to the subpixel. Can make the font appear lighter overall."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.fontAliasing.none',
                        "Disables font smoothing. Text will show with jagged sharp edges."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.fontAliasing.auto',
                        "Applies `default` or `antialiased` automatically based on the DPI of displays."
                    ))
                ],
                'included': isMacintosh
            },
            'workbench.settings.editor': {
                'type': 'string',
                'enum': ['ui', 'json'],
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'settings.editor.ui',
                        "Use the settings UI editor."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'settings.editor.json',
                        "Use the JSON file editor."
                    )),
                ],
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'settings.editor.desc',
                    "Determines which settings editor to use by default."
                )),
                'default': 'ui',
                'scope': 3
            },
            'workbench.hover.delay': {
                'type': 'number',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.hover.delay',
                    "Controls the delay in milliseconds after which the hover is shown for workbench items (ex. some extension provided tree view items). Already visible items may require a refresh before reflecting this setting change."
                )),
                'default': isMacintosh ? 1500 : 500,
                'minimum': 0
            },
            'workbench.reduceMotion': {
                type: 'string',
                description: ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'workbench.reduceMotion',
                    "Controls whether the workbench should render with fewer animations."
                )),
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.reduceMotion.on',
                        "Always render with reduced motion."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.reduceMotion.off',
                        "Do not render with reduced motion"
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'workbench.reduceMotion.auto',
                        "Render with reduced motion based on OS configuration."
                    )),
                ],
                default: 'auto',
                tags: ['accessibility'],
                enum: ['on', 'off', 'auto']
            },
            ["workbench.layoutControl.enabled" ]: {
                'type': 'boolean',
                'default': true,
                'markdownDescription': isWeb ?
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'layoutControlEnabledWeb',
                        "Controls whether the layout control in the title bar is shown."
                    )) :
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        { key: 'layoutControlEnabled', comment: ['{0}, {1} is a placeholder for a setting identifier.'] },
                        "Controls whether the layout control is shown in the custom title bar. This setting only has an effect when {0} is not set to {1}.",
                        '`#window.customTitleBarVisibility#`',
                        '`never`'
                    ))
            },
            'workbench.layoutControl.type': {
                'type': 'string',
                'enum': ['menu', 'toggles', 'both'],
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'layoutcontrol.type.menu',
                        "Shows a single button with a dropdown of layout options."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'layoutcontrol.type.toggles',
                        "Shows several buttons for toggling the visibility of the panels and side bar."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'layoutcontrol.type.both',
                        "Shows both the dropdown and toggle buttons."
                    )),
                ],
                'default': 'both',
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'layoutControlType',
                    "Controls whether the layout control in the custom title bar is displayed as a single menu button or with multiple UI toggles."
                )),
            },
            'workbench.tips.enabled': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'tips.enabled',
                    "When enabled, will show the watermark tips when no editor is open."
                ))
            },
        }
    });
    let windowTitleDescription = ( localizeWithPath(
        'vs/workbench/browser/workbench.contribution',
        'windowTitle',
        "Controls the window title based on the current context such as the opened workspace or active editor. Variables are substituted based on the context:"
    ));
    windowTitleDescription += '\n- ' + [
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'activeEditorShort',
            "`${activeEditorShort}`: the file name (e.g. myFile.txt)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'activeEditorMedium',
            "`${activeEditorMedium}`: the path of the file relative to the workspace folder (e.g. myFolder/myFileFolder/myFile.txt)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'activeEditorLong',
            "`${activeEditorLong}`: the full path of the file (e.g. /Users/Development/myFolder/myFileFolder/myFile.txt)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'activeFolderShort',
            "`${activeFolderShort}`: the name of the folder the file is contained in (e.g. myFileFolder)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'activeFolderMedium',
            "`${activeFolderMedium}`: the path of the folder the file is contained in, relative to the workspace folder (e.g. myFolder/myFileFolder)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'activeFolderLong',
            "`${activeFolderLong}`: the full path of the folder the file is contained in (e.g. /Users/Development/myFolder/myFileFolder)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'folderName',
            "`${folderName}`: name of the workspace folder the file is contained in (e.g. myFolder)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'folderPath',
            "`${folderPath}`: file path of the workspace folder the file is contained in (e.g. /Users/Development/myFolder)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'rootName',
            "`${rootName}`: name of the workspace with optional remote name and workspace indicator if applicable (e.g. myFolder, myRemoteFolder [SSH] or myWorkspace (Workspace))."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'rootNameShort',
            "`${rootNameShort}`: shortened name of the workspace without suffixes (e.g. myFolder, myRemoteFolder or myWorkspace)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'rootPath',
            "`${rootPath}`: file path of the opened workspace or folder (e.g. /Users/Development/myWorkspace)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'profileName',
            "`${profileName}`: name of the profile in which the workspace is opened (e.g. Data Science (Profile)). Ignored if default profile is used."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'appName',
            "`${appName}`: e.g. VS Code."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'remoteName',
            "`${remoteName}`: e.g. SSH"
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'dirty',
            "`${dirty}`: an indicator for when the active editor has unsaved changes."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'focusedView',
            "`${focusedView}`: the name of the view that is currently focused."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'activeRepositoryName',
            "`${activeRepositoryName}`: the name of the active repository (e.g. vscode)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'activeRepositoryBranchName',
            "`${activeRepositoryBranchName}`: the name of the active branch in the active repository (e.g. main)."
        )),
        ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'separator',
            "`${separator}`: a conditional separator (\" - \") that only shows when surrounded by variables with values or static text."
        ))
    ].join('\n- ');
    registry.registerConfiguration({
        'id': 'window',
        'order': 8,
        'title': ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'windowConfigurationTitle',
            "Window"
        )),
        'type': 'object',
        'properties': {
            'window.title': {
                'type': 'string',
                'default': defaultWindowTitle,
                'markdownDescription': windowTitleDescription
            },
            'window.titleSeparator': {
                'type': 'string',
                'default': defaultWindowTitleSeparator,
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    "window.titleSeparator",
                    "Separator used by {0}.",
                    '`#window.title#`'
                ))
            },
            ["window.commandCenter" ]: {
                type: 'boolean',
                default: true,
                markdownDescription: isWeb ?
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'window.commandCenterWeb',
                        "Show command launcher together with the window title."
                    )) :
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        { key: 'window.commandCenter', comment: ['{0}, {1} is a placeholder for a setting identifier.'] },
                        "Show command launcher together with the window title. This setting only has an effect when {0} is not set to {1}.",
                        '`#window.customTitleBarVisibility#`',
                        '`never`'
                    ))
            },
            'window.menuBarVisibility': {
                'type': 'string',
                'enum': ['classic', 'visible', 'toggle', 'hidden', 'compact'],
                'markdownEnumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'window.menuBarVisibility.classic',
                        "Menu is displayed at the top of the window and only hidden in full screen mode."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'window.menuBarVisibility.visible',
                        "Menu is always visible at the top of the window even in full screen mode."
                    )),
                    isMacintosh ?
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.menuBarVisibility.toggle.mac',
                            "Menu is hidden but can be displayed at the top of the window by executing the `Focus Application Menu` command."
                        )) :
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.menuBarVisibility.toggle',
                            "Menu is hidden but can be displayed at the top of the window via the Alt key."
                        )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'window.menuBarVisibility.hidden',
                        "Menu is always hidden."
                    )),
                    isWeb ?
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.menuBarVisibility.compact.web',
                            "Menu is displayed as a compact button in the side bar."
                        )) :
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            { key: 'window.menuBarVisibility.compact', comment: ['{0}, {1} is a placeholder for a setting identifier.'] },
                            "Menu is displayed as a compact button in the side bar. This value is ignored when {0} is {1}.",
                            '`#window.titleBarStyle#`',
                            '`native`'
                        ))
                ],
                'default': isWeb ? 'compact' : 'classic',
                'scope': 1 ,
                'markdownDescription': isMacintosh ?
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'menuBarVisibility.mac',
                        "Control the visibility of the menu bar. A setting of 'toggle' means that the menu bar is hidden and executing `Focus Application Menu` will show it. A setting of 'compact' will move the menu into the side bar."
                    )) :
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'menuBarVisibility',
                        "Control the visibility of the menu bar. A setting of 'toggle' means that the menu bar is hidden and a single press of the Alt key will show it. A setting of 'compact' will move the menu into the side bar."
                    )),
                'included': isWindows || isLinux || isWeb
            },
            'window.enableMenuBarMnemonics': {
                'type': 'boolean',
                'default': true,
                'scope': 1 ,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'enableMenuBarMnemonics',
                    "Controls whether the main menus can be opened via Alt-key shortcuts. Disabling mnemonics allows to bind these Alt-key shortcuts to editor commands instead."
                )),
                'included': isWindows || isLinux
            },
            'window.customMenuBarAltFocus': {
                'type': 'boolean',
                'default': true,
                'scope': 1 ,
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'customMenuBarAltFocus',
                    "Controls whether the menu bar will be focused by pressing the Alt-key. This setting has no effect on toggling the menu bar with the Alt-key."
                )),
                'included': isWindows || isLinux
            },
            'window.openFilesInNewWindow': {
                'type': 'string',
                'enum': ['on', 'off', 'default'],
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'window.openFilesInNewWindow.on',
                        "Files will open in a new window."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'window.openFilesInNewWindow.off',
                        "Files will open in the window with the files' folder open or the last active window."
                    )),
                    isMacintosh ?
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.openFilesInNewWindow.defaultMac',
                            "Files will open in the window with the files' folder open or the last active window unless opened via the Dock or from Finder."
                        )) :
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.openFilesInNewWindow.default',
                            "Files will open in a new window unless picked from within the application (e.g. via the File menu)."
                        ))
                ],
                'default': 'off',
                'scope': 1 ,
                'markdownDescription': isMacintosh ?
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'openFilesInNewWindowMac',
                        "Controls whether files should open in a new window when using a command line or file dialog.\nNote that there can still be cases where this setting is ignored (e.g. when using the `--new-window` or `--reuse-window` command line option)."
                    )) :
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'openFilesInNewWindow',
                        "Controls whether files should open in a new window when using a command line or file dialog.\nNote that there can still be cases where this setting is ignored (e.g. when using the `--new-window` or `--reuse-window` command line option)."
                    ))
            },
            'window.openFoldersInNewWindow': {
                'type': 'string',
                'enum': ['on', 'off', 'default'],
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'window.openFoldersInNewWindow.on',
                        "Folders will open in a new window."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'window.openFoldersInNewWindow.off',
                        "Folders will replace the last active window."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'window.openFoldersInNewWindow.default',
                        "Folders will open in a new window unless a folder is picked from within the application (e.g. via the File menu)."
                    ))
                ],
                'default': 'default',
                'scope': 1 ,
                'markdownDescription': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'openFoldersInNewWindow',
                    "Controls whether folders should open in a new window or replace the last active window.\nNote that there can still be cases where this setting is ignored (e.g. when using the `--new-window` or `--reuse-window` command line option)."
                ))
            },
            'window.confirmBeforeClose': {
                'type': 'string',
                'enum': ['always', 'keyboardOnly', 'never'],
                'enumDescriptions': [
                    isWeb ?
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.confirmBeforeClose.always.web',
                            "Always try to ask for confirmation. Note that browsers may still decide to close a tab or window without confirmation."
                        )) :
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.confirmBeforeClose.always',
                            "Always ask for confirmation."
                        )),
                    isWeb ?
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.confirmBeforeClose.keyboardOnly.web',
                            "Only ask for confirmation if a keybinding was used to close the window. Note that detection may not be possible in some cases."
                        )) :
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.confirmBeforeClose.keyboardOnly',
                            "Only ask for confirmation if a keybinding was used."
                        )),
                    isWeb ?
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.confirmBeforeClose.never.web',
                            "Never explicitly ask for confirmation unless data loss is imminent."
                        )) :
                        ( localizeWithPath(
                            'vs/workbench/browser/workbench.contribution',
                            'window.confirmBeforeClose.never',
                            "Never explicitly ask for confirmation."
                        ))
                ],
                'default': (isWeb && !isStandalone()) ? 'keyboardOnly' : 'never',
                'markdownDescription': isWeb ?
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'confirmBeforeCloseWeb',
                        "Controls whether to show a confirmation dialog before closing the browser tab or window. Note that even if enabled, browsers may still decide to close a tab or window without confirmation and that this setting is only a hint that may not work in all cases."
                    )) :
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'confirmBeforeClose',
                        "Controls whether to show a confirmation dialog before closing a window or quitting the application."
                    )),
                'scope': 1
            }
        }
    });
    registry.registerConfiguration({
        ...problemsConfigurationNodeBase,
        'properties': {
            'problems.visibility': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'problems.visibility',
                    "Controls whether the problems are visible throughout the editor and workbench."
                )),
            },
        }
    });
    registry.registerConfiguration({
        'id': 'zenMode',
        'order': 9,
        'title': ( localizeWithPath(
            'vs/workbench/browser/workbench.contribution',
            'zenModeConfigurationTitle',
            "Zen Mode"
        )),
        'type': 'object',
        'properties': {
            'zenMode.fullScreen': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'zenMode.fullScreen',
                    "Controls whether turning on Zen Mode also puts the workbench into full screen mode."
                ))
            },
            'zenMode.centerLayout': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'zenMode.centerLayout',
                    "Controls whether turning on Zen Mode also centers the layout."
                ))
            },
            'zenMode.showTabs': {
                'type': 'string',
                'enum': ['multiple', 'single', 'none'],
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'zenMode.showTabs',
                    "Controls whether turning on Zen Mode should show multiple editor tabs, a single editor tab, or hide the editor title area completely."
                )),
                'enumDescriptions': [
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'zenMode.showTabs.multiple',
                        "Each editor is displayed as a tab in the editor title area."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'zenMode.showTabs.single',
                        "The active editor is displayed as a single large tab in the editor title area."
                    )),
                    ( localizeWithPath(
                        'vs/workbench/browser/workbench.contribution',
                        'zenMode.showTabs.none',
                        "The editor title area is not displayed."
                    )),
                ],
                'default': 'multiple'
            },
            'zenMode.hideStatusBar': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'zenMode.hideStatusBar',
                    "Controls whether turning on Zen Mode also hides the status bar at the bottom of the workbench."
                ))
            },
            'zenMode.hideActivityBar': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'zenMode.hideActivityBar',
                    "Controls whether turning on Zen Mode also hides the activity bar either at the left or right of the workbench."
                ))
            },
            'zenMode.hideLineNumbers': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'zenMode.hideLineNumbers',
                    "Controls whether turning on Zen Mode also hides the editor line numbers."
                ))
            },
            'zenMode.restore': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'zenMode.restore',
                    "Controls whether a window should restore to Zen Mode if it was exited in Zen Mode."
                ))
            },
            'zenMode.silentNotifications': {
                'type': 'boolean',
                'default': true,
                'description': ( localizeWithPath(
                    'vs/workbench/browser/workbench.contribution',
                    'zenMode.silentNotifications',
                    "Controls whether notifications do not disturb mode should be enabled while in Zen Mode. If true, only error notifications will pop out."
                ))
            }
        }
    });
})();
( Registry.as(Extensions.ConfigurationMigration))
    .registerConfigurationMigrations([{
        key: 'workbench.activityBar.visible', migrateFn: (value) => {
            const result = [];
            if (value !== undefined) {
                result.push(['workbench.activityBar.visible', { value: undefined }]);
            }
            if (value === false) {
                result.push(["workbench.activityBar.location" , { value: "hidden"  }]);
            }
            return result;
        }
    }]);
( Registry.as(Extensions.ConfigurationMigration))
    .registerConfigurationMigrations([{
        key: "workbench.activityBar.location" , migrateFn: (value) => {
            const results = [];
            if (value === 'side') {
                results.push(["workbench.activityBar.location" , { value: "default"  }]);
            }
            return results;
        }
    }]);
( Registry.as(Extensions.ConfigurationMigration))
    .registerConfigurationMigrations([{
        key: 'workbench.editor.doubleClickTabToToggleEditorGroupSizes', migrateFn: (value) => {
            const results = [];
            if (typeof value === 'boolean') {
                value = value ? 'expand' : 'off';
                results.push(['workbench.editor.doubleClickTabToToggleEditorGroupSizes', { value }]);
            }
            return results;
        }
    }, {
        key: "workbench.editor.showTabs" , migrateFn: (value) => {
            const results = [];
            if (typeof value === 'boolean') {
                value = value ? "multiple"  : "single" ;
                results.push(["workbench.editor.showTabs" , { value }]);
            }
            return results;
        }
    }, {
        key: 'workbench.editor.tabCloseButton', migrateFn: (value) => {
            const result = [];
            if (value === 'left' || value === 'right') {
                result.push(['workbench.editor.tabActionLocation', { value }]);
            }
            else if (value === 'off') {
                result.push(['workbench.editor.tabActionCloseVisibility', { value: false }]);
            }
            return result;
        }
    }, {
        key: 'zenMode.hideTabs', migrateFn: (value) => {
            const result = [['zenMode.hideTabs', { value: undefined }]];
            if (value === true) {
                result.push(['zenMode.showTabs', { value: 'single' }]);
            }
            return result;
        }
    }]);
